@@ -9,7 +9,9 @@ from typing import IO, Any
 import click
 from pydantic.json import pydantic_encoder
 
+from unfolded.data_sdk.auth import authenticate as _authenticate
 from unfolded.data_sdk.data_sdk import DataSDK, RefreshToken
+from unfolded.data_sdk.environment import AuthEnvironment
 from unfolded.data_sdk.models import MediaType
 
 
@@ -25,6 +27,21 @@ def main() -> None:
     pass
 
 
+@click.command()
+@click.option(
+    "--credentials-dir",
+    type=PathType(dir_okay=True, file_okay=False, writable=True),
+    help="A directory on disk to use for storing credentials.",
+    default=Path("~/.config/unfolded/").expanduser(),
+    show_default=True,
+)
+def authenticate(credentials_dir: Path) -> None:
+    """Authenticate using OAuth2, storing credentials to disk"""
+    auth_env = AuthEnvironment.PRODUCTION
+    credentials = _authenticate(auth_env=auth_env)
+    DataSDK(refresh_token=credentials.refresh_token, credentials_dir=credentials_dir)
+
+
 @click.command()
 @click.option(
     "--organization/--no-organization",
@@ -188,6 +205,31 @@ def delete_dataset(dataset_id: str) -> None:
     click.echo("Dataset deleted.", file=sys.stderr)
 
 
+@click.command()
+@click.argument("map_id", type=str)
+@click.argument("dataset-to-replace-id", type=str)
+@click.argument("dataset-to-use-id", type=str)
+@click.option(
+    "--force",
+    is_flag=True,
+    callback=abort_if_false,
+    expose_value=False,
+    help="Force replace dataset (even if not compatible)",
+)
+def replace_dataset(
+    map_id: str, dataset_to_replace_id: str, dataset_to_use_id: str, force: bool = False
+) -> None:
+    """Replace one dataset associated with a map with another"""
+    data_sdk = DataSDK()
+    map_object = data_sdk.replace_dataset(
+        map_id,
+        dataset_to_replace=dataset_to_replace_id,
+        dataset_to_use=dataset_to_use_id,
+        force=force,
+    )
+    click.echo(map_object.json())
+
+
 @click.command()
 @click.option(
     "--organization/--no-organization",
@@ -348,6 +390,7 @@ def store_refresh_token(refresh_token: RefreshToken, credentials_dir: Path) -> N
     DataSDK(refresh_token=refresh_token, credentials_dir=credentials_dir)
 
 
+main.add_command(authenticate)
 main.add_command(list_datasets)
 main.add_command(get_dataset)
 main.add_command(download_dataset)
