@@ -13,7 +13,6 @@ from typing import (
     Dict,
     Iterable,
     List,
-    NewType,
     Optional,
     Sequence,
     Tuple,
@@ -44,6 +43,7 @@ from unfolded.data_sdk.models import (
     MapState,
     MapUpdateParams,
 )
+from unfolded.data_sdk.types import AccessToken, RefreshToken
 from unfolded.data_sdk.utils import (
     compress_fileobj,
     get_fileobj_length,
@@ -80,10 +80,6 @@ via Python or via the CLI do not need to pass a refresh token or access token.
 """
 
 
-AccessToken = NewType("AccessToken", str)
-RefreshToken = NewType("RefreshToken", str)
-
-
 class DataSDK:
     """Manage datasets and maps through the Unfolded Cloud."""
 
@@ -469,14 +465,12 @@ class DataSDK:
             if dataset:
                 # TODO: can updating a dataset also using name and description
                 # params?
-                r = requests.put(url, data=iterator, headers=headers)  # type: ignore
+                r = requests.put(url, data=iterator, headers=headers)
             else:
                 params = {"name": name}
                 if description:
                     params["description"] = description
-                r = requests.post(
-                    url, data=iterator, params=params, headers=headers  # type: ignore
-                )
+                r = requests.post(url, data=iterator, params=params, headers=headers)
 
         raise_for_status(r)
         return Dataset(**r.json())
@@ -556,6 +550,49 @@ class DataSDK:
         r = requests.delete(url, headers=self._headers)
         raise_for_status(r)
 
+    def replace_dataset(
+        self,
+        # pylint:disable = redefined-builtin
+        map: Union[Map, str, UUID],
+        dataset_to_replace: Union[Dataset, str, UUID],
+        dataset_to_use: Union[Dataset, str, UUID],
+        *,
+        force: bool = False,
+    ) -> Map:
+        """Replace a dataset within a Map with another one
+
+        This does not delete any datasets, it simply swaps one for another in a map.
+        Will error if the new dataset is not compatible with the old one (override with the force option).
+
+        Args:
+            map: the map to change. Can be a Map object or a string/UUID id.
+            dataset_to_replace: the dataset currently being used to be swapped out. Can be a Dataset object or a string/UUID id.
+            dataset_to_use: the new dataset to be added to the map. Can be a Dataset object or a string/UUID id.
+
+        Kwargs:
+            force: skip compatibility check and force replacement of datasets.
+
+        Returns:
+            The Map object that was operated upon.
+        """
+
+        if isinstance(map, Map):
+            map = map.id
+        if isinstance(dataset_to_replace, Dataset):
+            dataset_to_replace = dataset_to_replace.id
+        if isinstance(dataset_to_use, Dataset):
+            dataset_to_use = dataset_to_use.id
+
+        url = f"{self.base_url}/v1/maps/{map}/datasets/replace"
+        replace_dataset_payload = {
+            "datasetToReplaceId": str(dataset_to_replace),
+            "datasetToUseId": str(dataset_to_use),
+            "force": force,
+        }
+        r = requests.post(url, headers=self._headers, json=replace_dataset_payload)
+        raise_for_status(r)
+        return Map(**r.json())
+
     def list_maps(self, *, organization: bool = False) -> List[Map]:
         """List maps for authenticated user or organization
 
@@ -693,6 +730,8 @@ class DataSDK:
         Returns:
             The updated map record.
         """
+        if isinstance(map_id, Map):
+            map_id = map_id.id
 
         if not map_id:
             raise DataSDKError("Map id is required to perform the update")
@@ -750,7 +789,7 @@ class DataSDK:
 
     def enrich(
         self,
-        dataset: Union[pd.DataFrame, Dataset, UUID, str],
+        dataset: Union["pd.DataFrame", Dataset, UUID, str],
         source_id: UUID,
         source_column: Union[str, List[str]],
         *,
@@ -758,7 +797,7 @@ class DataSDK:
         lat_column: Optional[str] = None,
         lng_column: Optional[str] = None,
         time_column: Optional[str] = None,
-    ) -> pd.DataFrame:
+    ) -> "pd.DataFrame":
         """Enriches a dataset with a given enrichment source
 
         Enriches a dataset with a given enrichment source and column based on either a h3_column
@@ -779,6 +818,8 @@ class DataSDK:
         Returns:
             A pandas data frame with the resulting enriched dataset.
         """
+        if pd is None:
+            raise ImportError("Pandas required to use the enrich method.")
 
         if h3_column:
             index_columns = [h3_column]
@@ -840,7 +881,7 @@ class DataSDK:
         h3_column: Optional[str] = None,
         time_column: Optional[str] = None,
         time_interval: Optional[Union[Dict, TimeInterval]] = None,
-    ) -> pd.DataFrame:
+    ) -> "pd.DataFrame":
         """Extract data from a HexTile dataset
 
         Args:
@@ -857,6 +898,9 @@ class DataSDK:
         Returns:
             A pandas data frame with the extracted dataset.
         """
+        if pd is None:
+            raise ImportError("Pandas required to use the tile_extract method.")
+
         # Build and run the enrichment query
         query = Query()
         query = query.tile_extract(
@@ -917,7 +961,7 @@ class DataSDK:
 
         return Dataset(**r.json().get("item"))
 
-    def _query(self, query: Query) -> pd.DataFrame:
+    def _query(self, query: Query) -> "pd.DataFrame":
         """Runs a query using the Unfolded Query API
 
         Args:
@@ -926,6 +970,9 @@ class DataSDK:
         Returns:
             A pandas data frame with the result of the query.
         """
+        if pd is None:
+            raise ImportError("Pandas required to use the _query method.")
+
         url = f"{self.base_url}/v1/query"
         headers = {**self._headers, "Content-Type": "application/json"}
         r = requests.post(url, headers=headers, data=query.json())
@@ -1035,7 +1082,7 @@ class DataSDK:
         Returns:
             if True, should store credentials
         """
-        # See discussion in https://github.com/UnfoldedInc/platform/pull/2017 and
+        # See discussion in https://github.com/foursquare/unfolded-platform/pull/2017 and
         # related Jira ticket. In order to not break backwards compatibility, we need to
         # default existing users to `store_credentials=True`, or cycled refresh tokens
         # will not be persisted to disk. On the other hand, we want to default insecure
